#
# Name:         SA2xls.sh
# Author:       Frank Carvalho
# Date:         July 4th 2006
# This version: Based on revision 347 
#
# Invoke: . SA2xls.sh 2>&1
# Input:  
#
# This script parses a System Architect repository XML export and generates XPaths for every data structure element.
#
# Requires:
#    echoiso-8859-1.xsl
#    echoUTF-8.xsl
#    parser.xsl
#    canonical.xsl
#    SA2xls.xsl
#    saxon7.jar

#SCRIPTDIR=`which SA2xls.sh`
export SCRIPTDIR=`dirname $0`


PWD=`pwd`
export PWD

TARGETHOST="http://modelhost.ccta.dk"
export TARGETHOST

syntaxerror ()
{
  echo "Error - missing arguments"
  echo "  syntax:  . SAtoXSD.sh -source <source file> -target <target directory> [-pass <pass>] [-prefix <nsprefix>] [-ns <namespace>] [-pimp <shell script>]"
  echo "pass=0  # submit products to svn"
  echo "pass=1  # codepage"
  echo "pass=2  # ambiguious text"
  echo "pass=3  # data structure parser"
  echo "pass=4  # namespace fix"
  echo "pass=5  # XPath generation"
}

( # encapsulation of the entire script - to avoid hard exit of shell

# Read all shell script arguments

#export TARGETNAMESPACE='http://skat.dk/begrebsmodel/2006/09/07/'  # fase 1
#export TARGETPREFIX='tsi'                                         # fase 1

#export TARGETNAMESPACE='http://skat.dk/begrebsmodel/2008/01/01/'  # fase 2
#export TARGETPREFIX='fase2'                                       # fase 2

# Default namespace
export TARGETNAMESPACE='http://skat.dk/begrebsmodel/2009/01/15/'   # fase 2_1
export TARGETPREFIX='fase2_1'                                      # fase 2_1

for a in $*
do
  option=$1
  shift
  value=$1
  shift
  case $option in
  "-source") export SOURCE_FILE=$value;;
  "-target") export TARGET=$value;;
  "-pass")   export PASS=$value;;
  esac
done

if [ -z $SOURCE_FILE -o  -z $TARGET ]
then
  syntaxerror
  exit
fi

if [ ! -f $SOURCE_FILE ]
then 
  echo "File $SOURCE_FILE does not exist!"
  exit
fi

if [ -z $SCRIPTDIR ] # scriptdir does not exist 
then
  echo "Script directory not reckognised: $SCRIPTDIR"
  exit
fi

# Flags

export DEBUG=1                            # when defined all products of passes will be kept.

if [ $PASS ]
then
  case $PASS in
  0) PASS0=666; PASS1=1;;
  1) PASS1=1;;
  2) PASS2=2;;
  3) PASS3=3;;
  4) PASS4=4;;
  5) PASS5=5;;
  esac
else
  export PASS0=666 # Submit to version control - 666 because 0 is false.
  export PASS1=1   # codepage
  export PASS2=1   # ambiguious text
  export PASS3=1   # data structure parser
  export PASS4=1   # namespace fix
  export PASS5=1   # XPath generation
fi

theBase=`basename $TARGET`

# Retire previously generated directory in svn, if it exists.

TESTDIR="$TARGET/.svn"
export PREVREV="noRevision"
export REVISION="noRevision"

startTime=$(date +%s)

if [ $PASS0 ]
then
(
  cd `dirname $TARGET`
  rm -rf $TARGET
)
fi

if [ $PASS1 ]
then
  (
    mkdir $TARGET
    cd $TARGET
    mkdir tmp
  )
fi


cp $SOURCE_FILE $TARGET/.
cp `dirname $SOURCE_FILE`/saxml.dtd $TARGET
export SOURCE_FILE=$TARGET/`basename $SOURCE_FILE`

echo "============================ Pass0 done - deleted old directories"

echo "Generating XML schema, XSD, WSDL and RDF"
echo "  Namespace prefix     : $TARGETPREFIX"
echo "  Namespace            : $TARGETNAMESPACE"
echo "  Previous svn revision: $PREVREV"

if [ $PASS1 ]
then
java -Xmx900000000 -Xms900000000 -jar $SCRIPTDIR/saxon7.jar -o $TARGET/tmp/canonical-pass1.xml $SOURCE_FILE $SCRIPTDIR/xsl/echoUTF-8.xsl # This line will turn all encodings into UTF-8
fi

echo "============================ Pass1 done - UTF-8     "

if [ $PASS2 ]
then
cat $TARGET/tmp/canonical-pass1.xml | gawk 'BEGIN { AWKXMLENCODING="UTF-8";
        inquote=0;
      }
{
      gsub("&gt;","stoerre end",$0);
      gsub("&lt;","mindre end",$0);
      if(match($0,"SAObjMinorTypeName=\"Data Structure\""))
      {
	  inDS=1;
      }
      if(inDS==1 && match($0,"SAPrpName=\"Description\""))
      {
	  SAPrpValueStart  = match($0,"SAPrpValue=\"")+12
	  SAPrpValueLength = match(substr($0,SAPrpValueStart),"\"")-1
	  PrpValue         = substr($0,SAPrpValueStart,SAPrpValueLength)
          gsub("\t"," ",PrpValue); # All whitespace is space
          gsub("\n"," ",PrpValue); # All whitespace is space
          while(match(PrpValue,"  "))
          {
            gsub("  "," ",PrpValue); # Normalize space
          }

          remaining=PrpValue
	  trawl=""
          while(match(remaining,"*")>0) # saa laenge der er comments
          {
	    trawl=trawl substr(remaining,0,match(remaining,"*"))              # frem til stjerne
	    remaining=substr(remaining,match(remaining,"*")+1)                # resten er efter stjerne
            comment=substr(remaining,0,match(remaining,"*"))
            gsub(" ","",comment)                                              # rense for space
	    trawl=trawl comment                                               # tilføj renset frem til naeste stjerne
	    remaining=substr(remaining,match(remaining,"*")+1)                # resten er efter naeste stjerne
          }
          PrpValue=trawl remaining;
          
          #erstat alle spaces mellem to blokke [a-zæøåA-ZÆØÅ] med "+"
	  Command="echo \"" PrpValue "\" | sed \047s/\\\([a-z\346\370\345A-Z\306\330\3050-9]\\\)\040\\\([a-z\346\370\345A-Z\306\330\3050-9]\\\)/\\1+\\2/g\047 2>&1"
	  Command | getline Result
	  PrpValue=Result 
	  close(Command);

	  print substr($0,0,SAPrpValueStart-1) PrpValue substr($0,SAPrpValueStart+SAPrpValueLength) 
  
	  inDS=0;
	  next;
      }
      print $0
}' target=$TARGET tns=$TARGETNAMESPACE tnsprefix=$TARGETPREFIX > $TARGET/tmp/canonical-pass2.xml 2>/dev/null
fi

echo "============================ Pass2 done - Ambiguous texts replaced"

if [ $PASS3 ]
then
  java -Xmx900000000 -Xms900000000 -jar $SCRIPTDIR/saxon7.jar -o $TARGET/tmp/canonical-pass3.xml $TARGET/tmp/canonical-pass2.xml $SCRIPTDIR/xsl/parser.xsl target=$TARGET tns=$TARGETNAMESPACE tnsprefix=$TARGETPREFIX
fi

echo "============================ Pass3 done - Data Structures parsed"

if [ $PASS4 ]
then
cat $TARGET/tmp/canonical-pass3.xml | gawk 'BEGIN {  AWKXMLENCODING="UTF-8"
      }
{

  if(match($0,"<xs:schema"))
    {
      gsub(">"," xmlns:" tnsprefix "=\"" tns "\">",$0);
    }
  
  gsub("&#34;","",$0);
  gsub("&lt;","<",$0);
  gsub("&gt;",">",$0);

  #if(match($0,"<p-sequence"))
  #  {
  #    gsub(">"," name=\"Type" Sequence "\">",$0);
  #    Sequence++;
  #  }

  print $0;
}
END { }' target=$TARGET tns=$TARGETNAMESPACE tnsprefix=$TARGETPREFIX > $TARGET/tmp/canonical-pass4.xml
fi;

echo "============================ Pass4 done - Namespace fixed"

if [ $PASS5 ]
then
  java -Xmx900000000 -Xms900000000 -jar  $SCRIPTDIR/saxon9.jar -o $TARGET/tmp/canonical-pass5.xml $TARGET/tmp/canonical-pass4.xml $SCRIPTDIR/xsl/SA2xls.xsl # This line will turn all encodings into UTF-8
fi

echo "============================ Pass5 done - XML Schemas generated"


echo "============================ Logged all changes - elapsed $(($(date +%s) - startTime)) sec."
echo "Done"
) # encapsulation of the entire script - to avoid hard exit of shell
