<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.1" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:rdfs="http://www.w3.org/2000/01/rdf-schema#" xmlns:pars="pars" xmlns:tsi="http://toldskat.dk/xml/schemas/raplet/2005/01/01/" xmlns:xs="http://www.w3.org/2001/XMLSchema">

<!-- BUGS TO FIX:

- Andre Data Structure konstruktioner skal testes: Optional, Choice, Comment, Multiplicitet på element.

-->
	<xsl:param name="tns"/>
	<xsl:param name="tnsprefix"/>

	<xsl:template match="text()"/> <!-- To get rid of unwanted newlines, spaces etc. -->
	
	<xsl:output method="xml" indent="yes" omit-xml-declaration="no" encoding="UTF-8" media-type="text"/>
	<xsl:preserve-space elements=" "/>
	<xsl:variable name="OutputEncoding">iso-8859-1</xsl:variable>

	<xsl:template match="/SAEncyclopedia">
		<xsl:message>
			<xsl:text>Namespace=</xsl:text><xsl:value-of select="$tns"/><xsl:text>
</xsl:text>
			<xsl:text> Prefix=</xsl:text><xsl:value-of select="$tnsprefix"/><xsl:text>
</xsl:text>
		</xsl:message>

		<!-- <xsl:message terminate="no">
			<xsl:text>
Test: </xsl:text><xsl:value-of select="local-name(.)"/>
		</xsl:message> -->
		<xsl:element name="{name()}">
			<xsl:for-each select="@*">
				<xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute>
			</xsl:for-each>
			<xsl:apply-templates/>
		</xsl:element>
	</xsl:template>


	<xsl:template match="*">
		<xsl:element name="{name()}">
			<xsl:for-each select="@*">
				<xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute>
			</xsl:for-each>
			<xsl:apply-templates/>
		</xsl:element>
	</xsl:template>


<xsl:template match="SADefinition[@SAObjMinorTypeName='Data Domain' or @SAObjMinorTypeName='Data Element']/SAProperty[@SAPrpName='XSD Facet enumeration']">
	<xsl:variable name="DataStructure"><xsl:value-of select="@SAPrpValue"/></xsl:variable>
	<xsl:variable name="Enumerable">
		<xsl:call-template name="ParseEnumerable">
			<xsl:with-param name="RemainingDS">
				<xsl:value-of select="$DataStructure"/>
			</xsl:with-param>
		</xsl:call-template>
	</xsl:variable>

	<xsl:element name="{name()}">
		<xsl:for-each select="@*">
			<xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute>
		</xsl:for-each>

		<xsl:if test="string-length($DataStructure)>0">
			<xsl:element name="p-enumeration">
				<xsl:value-of select="normalize-space($Enumerable)"/>
			</xsl:element>

			<xsl:call-template name="ParseEnumeration">
				<xsl:with-param name="RemainingDS"><xsl:value-of select="substring-after($DataStructure,$Enumerable)"/></xsl:with-param>
			</xsl:call-template>
		</xsl:if>
	</xsl:element>
</xsl:template>


<xsl:template name="ParseEnumeration">
	<xsl:param name="RemainingDS"/>
		
	<xsl:variable name="Enumerable">
		<xsl:call-template name="ParseEnumerable">
			<xsl:with-param name="RemainingDS">
				<xsl:value-of select="$RemainingDS"/>
			</xsl:with-param>
		</xsl:call-template>
	</xsl:variable>

	<xsl:if test="string-length(translate($Enumerable,' ',''))>0">
		<xsl:element name="p-enumeration">
			<xsl:value-of select="normalize-space($Enumerable)"/>
		</xsl:element>

		<xsl:call-template name="ParseEnumeration">
			<xsl:with-param name="RemainingDS"><xsl:value-of select="substring-after($RemainingDS,$Enumerable)"/></xsl:with-param>
		</xsl:call-template>
	</xsl:if>

</xsl:template>

<xsl:template match="SADefinition[@SAObjMinorTypeName='Data Structure']/SAProperty[@SAPrpName='Description']">
	<xsl:variable name="DataStructure"><xsl:value-of select="./SAProperty[@SAPrpName='Description']/@SAPrpValue"/></xsl:variable>
   
	<xsl:element name="{name()}">
		<xsl:for-each select="@*">
			<xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute>
		</xsl:for-each>
		<xsl:apply-templates select="SALink"/>	
		<xsl:element name="p-block">
			<xsl:element name="p-sequence">
				<xsl:call-template name="Parse">
					<xsl:with-param name="RemainingDS"><xsl:value-of select="translate(@SAPrpValue,' ','')"/></xsl:with-param>
					<xsl:with-param name="nextIdpar">1</xsl:with-param>
				</xsl:call-template>
			</xsl:element>
		</xsl:element>
	</xsl:element>

</xsl:template>



	<xsl:template name="Parse">
		<xsl:param name="RemainingDS"/>
		<xsl:param name="minOccurs"/>
		<xsl:param name="nextIdpar"/>

		<xsl:param name="inComment"/>
		
		<xsl:variable name="Literal">
			<xsl:call-template name="ParseLiteral">
				<xsl:with-param name="RemainingDS">
					<xsl:value-of select="$RemainingDS"/>
				</xsl:with-param>
				<xsl:with-param name="inComment">
					<xsl:value-of select="$inComment"/>
				</xsl:with-param>
			</xsl:call-template>
		</xsl:variable>
		
		<xsl:variable name="NumberToken">
			<xsl:call-template name="ParseNumberToken">
				<xsl:with-param name="RemainingDS">
					<xsl:value-of select="$RemainingDS"/>
				</xsl:with-param>
			</xsl:call-template>
		</xsl:variable>
		
		<xsl:variable name="nextId">
			<xsl:value-of select="$nextIdpar"/>
		</xsl:variable>
		
<!--		<xsl:message terminate="no">
			<xsl:text>
Parse
	Literal='</xsl:text><xsl:value-of select="$Literal"/><xsl:text>'
	NumberToken='</xsl:text><xsl:value-of select="$NumberToken"/><xsl:text>'
	RemainingDS='</xsl:text><xsl:value-of select="$RemainingDS"/><xsl:text>'
	nextId='</xsl:text><xsl:value-of select="$nextId"/><xsl:text>'
	minOccurs='</xsl:text><xsl:value-of select="$minOccurs"/><xsl:text>'
	inComment='</xsl:text><xsl:value-of select="$inComment"/><xsl:text>'
			</xsl:text>
		</xsl:message> -->
		
		<xsl:choose>
			<xsl:when test="starts-with($RemainingDS,'}')">
				<!--<xsl:message>
					<xsl:text>}</xsl:text>
				</xsl:message> -->
				<xsl:text>
&lt;/p-sequence&gt;
</xsl:text>
				<xsl:variable name="Token">
					<xsl:value-of select="'}'"/>
				</xsl:variable>
				<xsl:variable name="maxOccur">
					<xsl:call-template name="ParseNumberToken">
						<xsl:with-param name="RemainingDS">
							<xsl:value-of select="substring-after($RemainingDS,$Token)"/>
						</xsl:with-param>
						<xsl:with-param name="Token">
							<xsl:value-of select="$Token"/>
						</xsl:with-param>
					</xsl:call-template>
				</xsl:variable>
				<xsl:if test="string-length($maxOccur)>1">
					<xsl:element name="p-maxOccurs">
						<xsl:value-of select="$maxOccur"/>
					</xsl:element>
				</xsl:if>
				<xsl:if test="$maxOccur=' '"> <!-- Unbounded max is default behaviour for sequences -->
					<xsl:element name="p-maxOccurs">
						<xsl:text>unbounded</xsl:text>
					</xsl:element>
				</xsl:if>
				<xsl:call-template name="ParseToken">
					<xsl:with-param name="RemainingDS">
						<xsl:value-of select="substring-after($RemainingDS,$Token)"/>
					</xsl:with-param>
					<xsl:with-param name="Token">
						<xsl:value-of select="$Token"/>
					</xsl:with-param>
					<xsl:with-param name="nextIdpar">
						<xsl:value-of select="$nextId"/>
					</xsl:with-param>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="starts-with($RemainingDS,']')">
				<!--<xsl:message>
					<xsl:text>]</xsl:text>
				</xsl:message>-->
				<xsl:text>
&lt;/p-sequence&gt;&lt;/p-choice&gt;</xsl:text>
				<xsl:if test="string-length($RemainingDS)>1">
					<xsl:call-template name="Parse">
						<xsl:with-param name="RemainingDS">
							<xsl:value-of select="substring($RemainingDS,2)"/>
						</xsl:with-param>
						<xsl:with-param name="nextIdpar">
							<xsl:value-of select="$nextId"/>
						</xsl:with-param>
					</xsl:call-template>
				</xsl:if>
			</xsl:when>
			<xsl:when test="starts-with($RemainingDS,'*')">
				<!-- <xsl:message>
					<xsl:text>*</xsl:text>
				</xsl:message> -->
				<!-- Previously an unbounded maxOccurs element was generated. Now * is just ignored -->
				<xsl:if test="not(string-length($inComment)>1)">
					<xsl:text>&lt;p-comment&gt;</xsl:text>
				</xsl:if>
				<xsl:if test="string-length($inComment)>1">
					<xsl:text>&lt;/p-comment&gt;</xsl:text>
				</xsl:if>

				<xsl:if test="string-length($RemainingDS)>1">
					<xsl:call-template name="Parse">
						<xsl:with-param name="RemainingDS">
							<xsl:value-of select="substring($RemainingDS,2)"/>
						</xsl:with-param>
						<xsl:with-param name="nextIdpar">
							<xsl:value-of select="$nextId"/>
						</xsl:with-param>
						<xsl:with-param name="inComment">
							<xsl:if test="not(string-length($inComment)>1)">
								<xsl:value-of select="22"/>
							</xsl:if>
						</xsl:with-param>
					</xsl:call-template>
				</xsl:if>
			</xsl:when>
			<xsl:when test="starts-with($RemainingDS,')')">
				<!-- <xsl:message>
					<xsl:text>)</xsl:text>
				</xsl:message> -->
				<xsl:text>
&lt;/p-optional&gt;
</xsl:text>
				<!-- <xsl:text> minOccurs="0"</xsl:text>-->
				<xsl:if test="string-length($RemainingDS)>1">
					<xsl:call-template name="Parse">
						<xsl:with-param name="RemainingDS">
							<xsl:value-of select="substring($RemainingDS,2)"/>
						</xsl:with-param>
						<xsl:with-param name="nextIdpar">
							<xsl:value-of select="$nextId"/>
						</xsl:with-param>
					</xsl:call-template>
				</xsl:if>
			</xsl:when>
			<xsl:when test="starts-with($RemainingDS,'{')">
				<!-- <xsl:message>
					<xsl:text>{    nextId=</xsl:text><xsl:value-of select="$nextId"/>
				</xsl:message>
				<xsl:message>
					<xsl:text>minOccurs=</xsl:text>
					<xsl:value-of select="$minOccurs"/>
				</xsl:message> -->
				<xsl:text>
&lt;p-sequence uniqueId="</xsl:text><xsl:value-of select="$nextId"/><xsl:text>"</xsl:text>
				<xsl:choose>
					<xsl:when test="string-length($minOccurs)>0 or $minOccurs='0'"> <!-- This last test circumvenes a bug in the string-length test. '0' tests to length 0 -->
						<xsl:text> minOccurs="</xsl:text>
						<xsl:value-of select="$minOccurs"/>
						<xsl:text>"</xsl:text>
					</xsl:when>
					<xsl:otherwise> <!-- Min 1 is default behaviour for sequences -->
						<xsl:text> minOccurs="1"</xsl:text>
					</xsl:otherwise>
				</xsl:choose>
				<xsl:text>&gt;
</xsl:text>
				<xsl:call-template name="ParseToken">
					<xsl:with-param name="RemainingDS">
						<xsl:value-of select="substring-after($RemainingDS,'{')"/>
					</xsl:with-param>
					<xsl:with-param name="Token">
						<xsl:value-of select="'{'"/>
					</xsl:with-param>
					<xsl:with-param name="nextIdpar">
						<xsl:value-of select="$nextId+1"/>  <!-- Notice that one is added here to generate unique type identifiers -->
					</xsl:with-param>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="starts-with($RemainingDS,'+') or starts-with($RemainingDS,' ') or starts-with($RemainingDS,'
') or starts-with($RemainingDS,'	')">
				<!-- <xsl:message>
					<xsl:text>+</xsl:text>
				</xsl:message> -->
				<xsl:element name="p-plus"/>
				<xsl:call-template name="ParseToken">
					<xsl:with-param name="RemainingDS">
						<xsl:value-of select="substring-after($RemainingDS,'+')"/>
					</xsl:with-param>
					<xsl:with-param name="Token">
						<xsl:value-of select="'+'"/>
					</xsl:with-param>
					<xsl:with-param name="nextIdpar">
						<xsl:value-of select="$nextId"/>
					</xsl:with-param>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="starts-with($RemainingDS,'(')">
				<!-- <xsl:message>
					<xsl:text>(</xsl:text>
				</xsl:message> -->
				<!-- <xsl:text> minOccurs="0"</xsl:text> -->
				<xsl:text>
&lt;p-optional&gt;
</xsl:text>
				<xsl:call-template name="ParseToken">
					<xsl:with-param name="RemainingDS">
						<xsl:value-of select="substring-after($RemainingDS,'(')"/>
					</xsl:with-param>
					<xsl:with-param name="Token">
						<xsl:value-of select="'('"/>
					</xsl:with-param>
					<xsl:with-param name="nextIdpar">
						<xsl:value-of select="$nextId"/>
					</xsl:with-param>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="starts-with($RemainingDS,'[')">
				<!--<xsl:message>
						<xsl:text>[</xsl:text>
					</xsl:message>-->
				<xsl:text>
&lt;p-choice&gt;&lt;p-sequence&gt;</xsl:text>
				<xsl:call-template name="ParseToken">
					<xsl:with-param name="RemainingDS">
						<xsl:value-of select="substring-after($RemainingDS,'[')"/>
					</xsl:with-param>
					<xsl:with-param name="Token">
						<xsl:value-of select="'['"/>
					</xsl:with-param>
					<xsl:with-param name="nextIdpar">
						<xsl:value-of select="$nextId"/>
					</xsl:with-param>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="starts-with($RemainingDS,'|')">
				<!-- <xsl:message>
					<xsl:text>|</xsl:text>
				</xsl:message> -->
				&lt;/p-sequence&gt;<xsl:element name="p-or"/>&lt;p-sequence&gt;
				<xsl:call-template name="ParseToken">
					<xsl:with-param name="RemainingDS">
						<xsl:value-of select="substring-after($RemainingDS,'|')"/>
					</xsl:with-param>
					<xsl:with-param name="Token">
						<xsl:value-of select="'|'"/>
					</xsl:with-param>
					<xsl:with-param name="nextIdpar">
						<xsl:value-of select="$nextId"/>
					</xsl:with-param>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="string-length($NumberToken)>1">
				<!-- <xsl:message>
					<xsl:text>NumberToken</xsl:text>
				</xsl:message> -->
				<xsl:if test="string-length(substring-after($RemainingDS,translate($NumberToken,' ','')))>0">
					<xsl:call-template name="Parse">
						<xsl:with-param name="RemainingDS">
							<xsl:value-of select="substring-after($RemainingDS,translate($NumberToken,' ',''))"/>
						</xsl:with-param>
						<xsl:with-param name="minOccurs">
							<xsl:value-of select="translate($NumberToken,' ','')"/>
						</xsl:with-param>
						<xsl:with-param name="nextIdpar">
							<xsl:value-of select="$nextId"/>
						</xsl:with-param>
					</xsl:call-template>
				</xsl:if>
			</xsl:when>
			<xsl:when test="string-length($Literal)>1">
				<!-- <xsl:message>
					<xsl:text>Literal</xsl:text>
				</xsl:message> -->
				<xsl:element name="p-element">
					<xsl:attribute name="name"><xsl:value-of select="translate($Literal,' ','')"/></xsl:attribute>
				</xsl:element>
				<xsl:if test="string-length(substring-after($RemainingDS,translate($Literal,' ','')))>0">
					<xsl:call-template name="Parse">
						<xsl:with-param name="RemainingDS">
							<xsl:value-of select="substring-after($RemainingDS,translate($Literal,' ',''))"/>
						</xsl:with-param>
						<xsl:with-param name="nextIdpar">
							<xsl:value-of select="$nextId"/>
						</xsl:with-param>
						<xsl:with-param name="inComment">
							<xsl:if test="string-length($inComment)>1">
								<xsl:value-of select="$inComment"/>
							</xsl:if>
						</xsl:with-param>
					</xsl:call-template>
				</xsl:if>
			</xsl:when>
			<xsl:otherwise>
				<xsl:text> </xsl:text>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	
	<xsl:template name="ParseToken">
		<xsl:param name="RemainingDS"/>
		<xsl:param name="Token"/>
		<xsl:param name="nextIdpar"/>
		<!-- <xsl:message><xsl:text>ParseToken </xsl:text><xsl:value-of select="$Token"/><xsl:text>, </xsl:text><xsl:value-of select="$RemainingDS"/></xsl:message> -->
		<xsl:variable name="Literal">
			<xsl:call-template name="ParseLiteral">
				<xsl:with-param name="RemainingDS">
					<xsl:value-of select="$RemainingDS"/>
				</xsl:with-param>
				<!-- <xsl:with-param name="RemainingDS"><xsl:value-of select="substring-after($RemainingDS,$Token)"/></xsl:with-param> -->
			</xsl:call-template>
		</xsl:variable>
		
		<xsl:variable name="nextId">
			<xsl:value-of select="$nextIdpar"/>
		</xsl:variable>

		<xsl:choose>
			<xsl:when test="string-length($Literal)>1">
				<!-- <xsl:message><xsl:text>string-length($Literal)>1</xsl:text></xsl:message> -->
				<xsl:element name="p-element">
					<xsl:attribute name="name"><xsl:value-of select="translate($Literal,' ','')"/></xsl:attribute>
				</xsl:element>
				<xsl:if test="string-length(substring-after($RemainingDS,translate($Literal,' ','')))>0">
					<xsl:call-template name="Parse">
						<xsl:with-param name="RemainingDS">
							<xsl:value-of select="substring-after($RemainingDS,translate($Literal,' ',''))"/>
						</xsl:with-param>
						<xsl:with-param name="nextIdpar">
							<xsl:value-of select="$nextId"/>
						</xsl:with-param>
					</xsl:call-template>
				</xsl:if>
			</xsl:when>
			<xsl:otherwise>
				<xsl:if test="string-length($RemainingDS)>0">
					<!-- <xsl:message><xsl:text>string-length($RemainingDS)>1, call parse on '</xsl:text><xsl:value-of select="$RemainingDS"/><xsl:text>'</xsl:text></xsl:message> -->
					<xsl:call-template name="Parse">
						<xsl:with-param name="RemainingDS">
							<xsl:value-of select="$RemainingDS"/>
						</xsl:with-param>
						<xsl:with-param name="nextIdpar">
							<xsl:value-of select="$nextId"/>
						</xsl:with-param>
					</xsl:call-template>
				</xsl:if>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<xsl:template name="ParseNumberToken">
		<xsl:param name="RemainingDS"/>
		<xsl:choose>
			<xsl:when test="starts-with($RemainingDS,'0') or starts-with($RemainingDS,'1') or starts-with($RemainingDS,'2') or starts-with($RemainingDS,'3') or starts-with($RemainingDS,'4') or starts-with($RemainingDS,'5') or starts-with($RemainingDS,'6') or starts-with($RemainingDS,'7') or starts-with($RemainingDS,'8') or starts-with($RemainingDS,'9')">
				<xsl:value-of select="substring($RemainingDS,1,1)"/>
				<xsl:call-template name="ParseNumberToken">
					<xsl:with-param name="RemainingDS">
						<xsl:value-of select="substring($RemainingDS,2)"/>
					</xsl:with-param>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:text> </xsl:text>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	
	<xsl:template name="ParseLiteral">
		<xsl:param name="RemainingDS"/>
		<xsl:param name="inComment"/>
		<!-- <xsl:message><xsl:text>ParseLiteral </xsl:text><xsl:value-of select="$RemainingDS"/></xsl:message> -->
		<xsl:choose>
			<xsl:when test="(not(starts-with($RemainingDS,'0'))) and (not(starts-with($RemainingDS,'1'))) and (not(starts-with($RemainingDS,'2'))) and (not(starts-with($RemainingDS,'3'))) and (not(starts-with($RemainingDS,'4'))) and (not(starts-with($RemainingDS,'5'))) and (not(starts-with($RemainingDS,'6'))) and (not(starts-with($RemainingDS,'7'))) and (not(starts-with($RemainingDS,'8'))) and (not(starts-with($RemainingDS,'9'))) and (not(starts-with($RemainingDS,'{'))) and (not(starts-with($RemainingDS,'}'))) and (not(starts-with($RemainingDS,'['))) and (not(starts-with($RemainingDS,']'))) and (not(starts-with($RemainingDS,'|'))) and (not(starts-with($RemainingDS,'('))) and (not(starts-with($RemainingDS,')'))) and (string-length(inComment)>0 or not(starts-with($RemainingDS,'+'))) and (not(starts-with($RemainingDS,'*')))">
				<xsl:value-of select="substring($RemainingDS,1,1)"/>
				<xsl:if test="string-length(substring($RemainingDS,2))>0">
					<xsl:call-template name="ParseLiteral2">
						<xsl:with-param name="RemainingDS">
							<xsl:choose>
								<xsl:when test="string-length(inComment)>0 and starts-with($RemainingDS,'+')">
									<xsl:value-of select="substring($RemainingDS,3)"/>
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of select="substring($RemainingDS,2)"/>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:with-param>
						<xsl:with-param name="inComment">
							<xsl:if test="string-length($inComment)>1">
								<xsl:value-of select="$inComment"/>
							</xsl:if>
						</xsl:with-param>
					</xsl:call-template>
				</xsl:if>
			</xsl:when>
			<xsl:otherwise>
				<xsl:text> </xsl:text>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<xsl:template name="ParseLiteral2">
		<xsl:param name="RemainingDS"/>
		<xsl:param name="inComment"/>
		<!-- <xsl:message><xsl:text>ParseLiteral2 </xsl:text><xsl:value-of select="$RemainingDS"/></xsl:message> -->
		<xsl:choose>
			<xsl:when test="(not(starts-with($RemainingDS,'{'))) and (not(starts-with($RemainingDS,'}'))) and (not(starts-with($RemainingDS,'['))) and (not(starts-with($RemainingDS,']'))) and (not(starts-with($RemainingDS,'|'))) and (not(starts-with($RemainingDS,'('))) and (not(starts-with($RemainingDS,')'))) and (string-length(inComment)>0 or not(starts-with($RemainingDS,'+'))) and (not(starts-with($RemainingDS,'*')))">
				<xsl:value-of select="substring($RemainingDS,1,1)"/>
				<xsl:if test="string-length(substring($RemainingDS,2))>0">
					<xsl:call-template name="ParseLiteral2">
						<xsl:with-param name="RemainingDS">
							<xsl:choose>
								<xsl:when test="string-length(inComment)>0 and starts-with($RemainingDS,'+')">
									<xsl:value-of select="substring($RemainingDS,3)"/>
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of select="substring($RemainingDS,2)"/>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:with-param>
						<xsl:with-param name="inComment">
							<xsl:if test="string-length($inComment)>1">
								<xsl:value-of select="$inComment"/>
							</xsl:if>
						</xsl:with-param>
					</xsl:call-template>
				</xsl:if>
			</xsl:when>
			<xsl:otherwise>
				<xsl:text> </xsl:text>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="ParseEnumerable">
		<xsl:param name="RemainingDS"/>
		<!-- <xsl:message><xsl:text>ParseLiteral </xsl:text><xsl:value-of select="$RemainingDS"/></xsl:message> -->
		<xsl:choose> <!--  (not(starts-with($RemainingDS,'0'))) and (not(starts-with($RemainingDS,'1'))) and (not(starts-with($RemainingDS,'2'))) and (not(starts-with($RemainingDS,'3'))) and (not(starts-with($RemainingDS,'4'))) and (not(starts-with($RemainingDS,'5'))) and (not(starts-with($RemainingDS,'6'))) and (not(starts-with($RemainingDS,'7'))) and (not(starts-with($RemainingDS,'8'))) and (not(starts-with($RemainingDS,'9'))) and -->
			<xsl:when test="(not(starts-with($RemainingDS,'{'))) and (not(starts-with($RemainingDS,'}'))) and (not(starts-with($RemainingDS,'['))) and (not(starts-with($RemainingDS,']'))) and (not(starts-with($RemainingDS,'|'))) and (not(starts-with($RemainingDS,'('))) and (not(starts-with($RemainingDS,')'))) and (not(starts-with($RemainingDS,'+'))) and (not(starts-with($RemainingDS,'*'))) and (not(starts-with($RemainingDS,' ')))">
				<xsl:value-of select="substring($RemainingDS,1,1)"/>
				<xsl:if test="string-length(substring($RemainingDS,2))>0">
					<xsl:call-template name="ParseEnumerable2">
						<xsl:with-param name="RemainingDS">
							<xsl:value-of select="substring($RemainingDS,2)"/>
						</xsl:with-param>
					</xsl:call-template>
				</xsl:if>
			</xsl:when>
			<xsl:otherwise>
				<xsl:text> </xsl:text>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<xsl:template name="ParseEnumerable2">
		<xsl:param name="RemainingDS"/>
		<!-- <xsl:message><xsl:text>ParseLiteral2 </xsl:text><xsl:value-of select="$RemainingDS"/></xsl:message> -->
		<xsl:choose>
			<xsl:when test="(not(starts-with($RemainingDS,'{'))) and (not(starts-with($RemainingDS,'}'))) and (not(starts-with($RemainingDS,'['))) and (not(starts-with($RemainingDS,']'))) and (not(starts-with($RemainingDS,'|'))) and (not(starts-with($RemainingDS,'('))) and (not(starts-with($RemainingDS,')'))) and (not(starts-with($RemainingDS,'+'))) and (not(starts-with($RemainingDS,'*'))) and (not(starts-with($RemainingDS,' ')))">
				<xsl:value-of select="substring($RemainingDS,1,1)"/>
				<xsl:if test="string-length(substring($RemainingDS,2))>0">
					<xsl:call-template name="ParseEnumerable2">
						<xsl:with-param name="RemainingDS">
							<xsl:value-of select="substring($RemainingDS,2)"/>
						</xsl:with-param>
					</xsl:call-template>
				</xsl:if>
			</xsl:when>
			<xsl:otherwise>
				<xsl:text> </xsl:text>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

</xsl:stylesheet>
